# Example Koji callback
# Copyright (c) 2016 CS2C Inc.
# This callback sign key for tag builds
#
# Authors:
#     fei.luo <fei.luo@cs2c.com.cn>

from koji.plugin import callbacks, callback, ignore_error
import logging
from koji.context import context
import sys
import koji
import ConfigParser
import xmlrpclib
import os

# XXX - have to import kojihub for mktask
sys.path.insert(0, '/usr/share/koji-hub/')
from kojihub import _fetchSingle, get_rpm, get_build

CONFIG_FILE = '/etc/koji-hub/plugins/sign.conf'

koji.add_file_logger("koji.plugin.sign", "/var/log/koji_plugin_sign.log")
logger = logging.getLogger('koji.plugin.sign')
logger.setLevel(logging.DEBUG)

handle = []


class Options: pass


def get_session():
    # load local config
    config = ConfigParser.ConfigParser()
    config.read(CONFIG_FILE)

    options = Options()
    if config.has_section('sign'):
        server = config.get('sign', 'server')
        cert = config.get('sign', 'cert')
        ca = config.get('sign', 'ca')
        serverca = config.get('sign', 'serverca')
        user = config.get('sign', 'user')
        password = config.get('sign', 'password')

        defaults = {
            'server': server,
            'cert': cert,
            'ca': ca,
            'serverca': serverca,
            'user': user,
            'password': password
        }

        for name, value in defaults.items():
            if getattr(options, name, None) is None:
                setattr(options, name, value)

        # start a session and login
        session = koji.ClientSession(options.server, defaults)
        if options.user:
            try:
                # authenticate using user/password
                session.login()
            except koji.AuthError:
                quit("Error: Unable to log in. Bad credentials?")
            except xmlrpclib.ProtocolError:
                quit("Error: Unable to connect to server %s" % (options.server))
        elif os.path.isfile(options.cert):
            try:
                # authenticate using SSL client certificates
                session.ssl_login(options.cert, options.ca,
                                  options.serverca)
            except koji.AuthError, e:
                quit("Error: Unable to log in: %s" % e)
            except xmlrpclib.ProtocolError:
                quit("Error: Unable to connect to server %s" % (options.server))

        # make sure it works
        try:
            ret = session.echo("OK")
        except xmlrpclib.ProtocolError:
            quit("Error: Unable to connect to server %s" % (options.server))
        if ret != ["OK"]:
            quit("Error: incorrect server response: %r" % (ret))
        return session


def check_rpm_sign_in_db(rpms, sigkey):
    # make sure we have it in the db
    need_handle = []
    for an_rpm in rpms:
        # XXX - still not sure if this is the right restriction
        rinfo = get_rpm(an_rpm, strict=True)
        if rinfo['external_repo_id']:
            raise koji.GenericError, "Not an internal rpm: %s (from %s)" \
                                     % (an_rpm, rinfo['external_repo_name'])
        binfo = get_build(rinfo['build_id'])
        nvra = "%(name)s-%(version)s-%(release)s.%(arch)s" % rinfo
        builddir = koji.pathinfo.build(binfo)
        rpm_path = "%s/%s" % (builddir, koji.pathinfo.rpm(rinfo))
        if not os.path.exists(rpm_path):
            raise koji.GenericError, "No such path: %s" % rpm_path
        if not os.path.isfile(rpm_path):
            raise koji.GenericError, "Not a regular file: %s" % rpm_path

        rpm_id = an_rpm['id']
        q = """SELECT sighash FROM rpmsigs WHERE rpm_id=%(rpm_id)i AND sigkey=%(sigkey)s"""

        row = _fetchSingle(q, locals())
        if not row:
            need_handle.append(nvra)
    return need_handle


def resign_build_by_id(session, build_id, sigkey):
    """
    Resign build through build id
    :param self:
    :param build_id:
    :param sigkey:
    :return:
    """
    rpms = session.listRPMs(int(build_id))
    hand = check_rpm_sign_in_db(rpms, sigkey)
    handle.extend(hand)
    new_handle = list(set(handle))
    if len(new_handle) > 0:
        print 'No cached signature for package'
        print os.linesep.join(handle)
        return
    else:
        for rpm in rpms:
            session.writeSignedRPM(rpm['id'], sigkey)
            print 'Write %s successfully' % rpm['nvr']


@callback(*callbacks.keys())
@ignore_error
def resign_tag_build(*args, **kws):
    """
    resign all builds attach the tag
    :param args:
    :param kws:
    :return:
    """
    context.session.assertPerm('admin')
    tag = kws['tag']
    key = kws['key']
    session = get_session()
    builds = session.listTagged(tag, inherit=False)

    for build in builds:
        # logger.warning('Resigned nvr %s , id %s ' % (build['nvr'], build['build_id']), tag, key, exc_info=True)
        print 'Resigned nvr %s , id %s ' % (build['nvr'], build['build_id']), tag, key
        resign_build_by_id(session, build['build_id'], key)
    session.logout()


resign_tag_build.exported = True
